"""
A Python "serializer". Doesn't do much serializing per se -- just converts to
and from basic Python data types (lists, dicts, strings, etc.). Useful as a basis for
other serializers.
"""
# Avoid shadowing the standard library json module
from __future__ import absolute_import
#from __future__ import unicode_literals

# Python
import datetime
import decimal

# Django
from django.conf import settings
from django.core.serializers import base
from django.core.serializers.base import DeserializationError
from django.db import models, DEFAULT_DB_ALIAS
from django.utils.encoding import smart_text, is_protected_type, smart_unicode
from django.utils import simplejson
from django.utils import six
from django.utils.timezone import is_aware

def _get_model(model_identifier):
    """
    Helper to look up a model from an "app_label.module_name" string.
    """
    try:
        Model = models.get_model(*model_identifier.split("."))
    except TypeError:
        Model = None
    if Model is None:
        raise base.DeserializationError("Invalid model identifier: '%s'" % model_identifier)
    return Model
    
class JsonSerializer(base.Serializer):
    """Serialize a queryset to JSON. Copy of serializer from Django 1.5."""

    internal_use_only = False

    def serialize(self, queryset, **options):
        """Serialize a queryset."""
        self.options = options
        self.stream = options.pop('stream', six.StringIO())
        self.selected_fields = options.pop('fields', None)
        self.use_natural_keys = options.pop('use_natural_keys', True)

        self.start_serialization()
        self.first = True
        for obj in queryset:
            self.start_object(obj)
            # Use the concrete parent class' _meta instead of the object's _meta
            # This is to avoid local_fields problems for proxy models. Refs #17717.
            concrete_model = obj._meta.concrete_model
            for field in concrete_model._meta.local_fields:
                if field.serialize:
                    if field.rel is None:
                        if self.selected_fields is None or field.attname in self.selected_fields:
                            self.handle_field(obj, field)
                    else:
                        if self.selected_fields is None or field.attname[:-3] in self.selected_fields:
                            self.handle_fk_field(obj, field)
            for field in concrete_model._meta.many_to_many:
                if field.serialize:
                    if self.selected_fields is None or field.attname in self.selected_fields:
                        self.handle_m2m_field(obj, field)
            self.end_object(obj)
            if self.first:
                self.first = False
        self.end_serialization()
        return self.getvalue()

    def start_serialization(self):
        if simplejson.__version__.split('.') >= ['2', '1', '3']:
            # Use JS strings to represent Python Decimal instances (ticket #16850)
            self.options.update({'use_decimal': False})
        self._current = None
        self.json_kwargs = self.options.copy()
        self.json_kwargs.pop('stream', None)
        self.json_kwargs.pop('fields', None)
        self.stream.write('[')

    def end_serialization(self):
        if self.options.get('indent'):
            self.stream.write('\n')
        self.stream.write(']')
        if self.options.get('indent'):
            self.stream.write('\n')

    def start_object(self, obj):
        self._current = {}

    def end_object(self, obj):
        # self._current has the field data
        indent = self.options.get('indent')
        if not self.first:
            self.stream.write(',')
            if not indent:
                self.stream.write(' ')
        if indent:
            self.stream.write('\n')
        simplejson.dump(self.get_dump_object(obj), self.stream,
                        cls=DjangoJSONEncoder, **self.json_kwargs)
        self._current = None

    def get_dump_object(self, obj):
        obj_dump = {
            'pk': smart_text(obj._get_pk_val(), strings_only=True),
            'model': smart_text(obj._meta),
            'fields': self._current,
        }
        if self.use_natural_keys and hasattr(obj, 'natural_key'):
            obj_dump['nk'] = obj.natural_key()
        return obj_dump

    def handle_field(self, obj, field):
        value = field._get_val_from_obj(obj)
        # Protected types (i.e., primitives like None, numbers, dates,
        # and Decimals) are passed through as is. All other values are
        # converted to string first.
        if is_protected_type(value):
            self._current[field.name] = value
        else:
            self._current[field.name] = field.value_to_string(obj)

    def handle_fk_field(self, obj, field):
        if self.use_natural_keys and hasattr(field.rel.to, 'natural_key'):
            related = getattr(obj, field.name)
            if related:
                value = related.natural_key(),
            else:
                value = None
        else:
            value = getattr(obj, field.get_attname())
        self._current[field.name] = value

    def handle_m2m_field(self, obj, field):
        if field.rel.through._meta.auto_created:
            if self.use_natural_keys and hasattr(field.rel.to, 'natural_key'):
                m2m_value = lambda value: value.natural_key()
            else:
                m2m_value = lambda value: smart_text(value._get_pk_val(), strings_only=True)
            self._current[field.name] = [m2m_value(related)
                               for related in getattr(obj, field.name).iterator()]

    def getvalue(self):
        if callable(getattr(self.stream, 'getvalue', None)):
            return self.stream.getvalue()


def PythonDeserializer(object_list, **options):
    """
    Deserialize simple Python objects back into Django ORM instances.

    It's expected that you pass the Python objects themselves (instead of a
    stream or a string) to the constructor
    """
    db = options.pop('using', DEFAULT_DB_ALIAS)
    ignore = options.pop('ignorenonexistent', False)

    models.get_apps()
    for d in object_list:
        # Look up the model and starting build a dict of data for it.
        Model = _get_model(d["model"])
        data = {Model._meta.pk.attname: Model._meta.pk.to_python(d["pk"])}
        m2m_data = {}
        model_fields = Model._meta.get_all_field_names()

        # Handle each field
        for (field_name, field_value) in six.iteritems(d["fields"]):

            if ignore and field_name not in model_fields:
                # skip fields no longer on model
                continue

            if isinstance(field_value, str):
                field_value = smart_text(field_value, options.get("encoding", settings.DEFAULT_CHARSET), strings_only=True)

            field = Model._meta.get_field(field_name)

            # Handle M2M relations
            if field.rel and isinstance(field.rel, models.ManyToManyRel):
                if hasattr(field.rel.to._default_manager, 'get_by_natural_key'):
                    def m2m_convert(value):
                        if hasattr(value, '__iter__') and not isinstance(value, six.text_type):
                            return field.rel.to._default_manager.db_manager(db).get_by_natural_key(*value).pk
                        else:
                            return smart_text(field.rel.to._meta.pk.to_python(value))
                else:
                    m2m_convert = lambda v: smart_text(field.rel.to._meta.pk.to_python(v))
                m2m_data[field.name] = [m2m_convert(pk) for pk in field_value]

            # Handle FK fields
            elif field.rel and isinstance(field.rel, models.ManyToOneRel):
                if field_value is not None:
                    if hasattr(field.rel.to._default_manager, 'get_by_natural_key'):
                        if hasattr(field_value, '__iter__') and not isinstance(field_value, six.text_type):
                            obj = field.rel.to._default_manager.db_manager(db).get_by_natural_key(*field_value)
                            value = getattr(obj, field.rel.field_name)
                            # If this is a natural foreign key to an object that
                            # has a FK/O2O as the foreign key, use the FK value
                            if field.rel.to._meta.pk.rel:
                                value = value.pk
                        else:
                            value = field.rel.to._meta.get_field(field.rel.field_name).to_python(field_value)
                        data[field.attname] = value
                    else:
                        data[field.attname] = field.rel.to._meta.get_field(field.rel.field_name).to_python(field_value)
                else:
                    data[field.attname] = None

            # Handle all other fields
            else:
                data[field.name] = field.to_python(field_value)

        yield base.DeserializedObject(Model(**data), m2m_data)



def Deserializer(stream_or_string, **options):
    """
    Deserialize a stream or string of JSON data.
    """
    if not isinstance(stream_or_string, (bytes, six.string_types)):
        stream_or_string = stream_or_string.read()
    if isinstance(stream_or_string, bytes):
        stream_or_string = stream_or_string.decode('utf-8')
    try:
        objects = simplejson.loads(stream_or_string)
        for obj in PythonDeserializer(objects, **options):
            yield obj
    except GeneratorExit:
        raise
    except Exception, e:
        # Map to deserializer error
        raise DeserializationError(e)


class DjangoJSONEncoder(simplejson.JSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date/time and decimal types.
    """
    def default(self, o):
        # See "Date Time String Format" in the ECMA-262 specification.
        if isinstance(o, datetime.datetime):
            r = o.isoformat()
            if o.microsecond:
                r = r[:23] + r[26:]
            if r.endswith('+00:00'):
                r = r[:-6] + 'Z'
            return r
        elif isinstance(o, datetime.date):
            return o.isoformat()
        elif isinstance(o, datetime.time):
            if is_aware(o):
                raise ValueError("JSON can't represent timezone-aware times.")
            r = o.isoformat()
            if o.microsecond:
                r = r[:12]
            return r
        elif isinstance(o, decimal.Decimal):
            return str(o)
        else:
            return super(DjangoJSONEncoder, self).default(o)



# Django
from django.db import models

class Document(models.Model):
    """Test model with a file field."""

    title = models.CharField(max_length=100)
    doc = models.FileField(upload_to='documents')
    # Separate date/time fields so we have those types for testing site_dump.
    created_date = models.DateField(auto_now_add=True)
    created_time = models.TimeField(auto_now_add=True)
    created_by = models.ForeignKey('auth.User', related_name='documents',
                                   blank=True, null=True, default=None)

class Photo(models.Model):
    """Test model with an image field."""

    caption = models.CharField(max_length=100)
    image = models.ImageField(upload_to='photos')
    # Separate date/time fields so we have those types for testing site_dump.
    created_date = models.DateField(auto_now_add=True)
    created_time = models.TimeField(auto_now_add=True)
    created_by = models.ForeignKey('auth.User', related_name='photos',
                                   blank=True, null=True, default=None)



# Python
import datetime
from optparse import make_option
import os
import shutil
import tempfile
import zipfile

# Django
from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand, CommandError
from django.core.management.commands.dumpdata import sort_dependencies
from django.db import models
from django.db import router, DEFAULT_DB_ALIAS
from django.db.models import get_apps, get_models, get_model, get_app
from django.utils.datastructures import SortedDict

# Django-Site-Utils
from site_utils.serializers import JsonSerializer

class Command(BaseCommand):
    """Dump the contents of the entire site, including media files."""

    option_list = BaseCommand.option_list + (
        make_option('-o', '--output', default=None, dest='output',
            help='Specifies the output archive (ZIP) filename.'),
        make_option('--database', action='store', dest='database',
            default=DEFAULT_DB_ALIAS, help='Nominates a specific database to '
                'dump fixtures from. Defaults to the "default" database.'),
        make_option('-e', '--exclude', dest='exclude', action='append',
            default=[], help='An appname or appname.ModelName to exclude (use '
            'multiple --exclude to exclude multiple apps/models).'),
        make_option('-a', '--all', action='store_true', dest='use_base_manager',
            default=True, help='Use Django\'s base manager to dump all models '
            'stored in the database, including those that would otherwise be '
            'filtered or modified by a custom manager.'),
    )
    help = ('Dump the contents of the entire site, including media files, to '
            'an archive containing a fixture of the given format.')
    args = '[appname appname.ModelName ...]'

    def build_app_dict(self, app_labels=[], excludes=[]):
        """Build dictionary of apps and models to dump."""
        # Determine excluded apps and models.
        excluded_apps = set()
        excluded_models = set()
        for exclude in excludes:
            if '.' in exclude:
                app_label, model_name = exclude.split('.', 1)
                model_obj = get_model(app_label, model_name)
                if not model_obj:
                    raise CommandError('Unknown model in excludes: %s' % exclude)
                excluded_models.add(model_obj)
            else:
                try:
                    app_obj = get_app(exclude)
                    excluded_apps.add(app_obj)
                except ImproperlyConfigured:
                    raise CommandError('Unknown app in excludes: %s' % exclude)
        # Build dictionary of apps (as keys) and a list of models (as values).
        if not app_labels:
            app_labels = [app.__name__.split('.')[-2] for app in get_apps()]
        app_list = SortedDict()
        for label in app_labels:
            try:
                # App with model name.
                app_label, model_label = label.split('.')
                try:
                    app = get_app(app_label)
                except ImproperlyConfigured:
                    raise CommandError('Unknown application: %s' % app_label)
                if app in excluded_apps:
                    continue
                model = get_model(app_label, model_label)
                if model is None:
                    raise CommandError('Unknown model: %s.%s' % (app_label, model_label))
                if model in excluded_models:
                    continue
                if app in app_list.keys():
                    if app_list[app] and model not in app_list[app]:
                        app_list[app].append(model)
                else:
                    app_list[app] = [model]
            except ValueError:
                # App label only, with no model qualifier.
                app_label = label
                try:
                    app = get_app(app_label)
                except ImproperlyConfigured:
                    raise CommandError('Unknown application: %s' % app_label)
                if app in excluded_apps:
                    continue
                if app not in app_list.keys():
                    app_list[app] = []
                for model in get_models(app):
                    if model in excluded_models:
                        continue
                    if model not in app_list[app]:
                        app_list[app].append(model)
        return app_list

    def save_file_field(self, obj, field, archive):
        """Save file field data to the given archive."""
        src = getattr(obj, field.name)
        if src.name:
            try:
                tf_path = None
                try:
                    file_path = src.path
                except NotImplementedError:
                    file_path = None
                if file_path is None:
                    src.open('rb')
                    tf_handle, tf_path = tempfile.mkstemp()
                    tf = os.fdopen(tf_handle, 'wb')
                    shutil.copyfileobj(src, tf)
                    src.close()
                    tf.close()
                    file_path = tf_path
                zip_path = 'site_media/%s' % src.name
                zip_path = zip_path.encode('ascii')
                archive.write(file_path, zip_path)
            finally:
                try:
                    if tf_path:
                        os.remove(tf_path)
                except:
                    pass

    def iter_objects(self, app_dict, using, use_base_manager=False, archive=None):
        """Iterate over all models from the given app dictionary."""
        for model in sort_dependencies(app_dict.items()):
            if not model._meta.proxy and router.allow_syncdb(using, model):
                file_fields = set()
                for field in model._meta.fields:
                    if isinstance(field, models.FileField):
                        file_fields.add(field)
                if use_base_manager:
                    objects = model._base_manager
                else:
                    objects = model._default_manager
                for obj in objects.using(using).order_by(model._meta.pk.name).iterator():
                    yield obj
                    for field in file_fields:
                        self.save_file_field(obj, field, archive)

    def handle(self, *app_labels, **options):
        """Handle the site_dump management command."""
        verbosity = int(options.get('verbosity', 1))
        output = options.get('output', None)
        if output is None:
            output = 'site_dump_%s.zip' % datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
        using = options.get('database')
        excludes = options.get('exclude')
        show_traceback = options.get('traceback')
        use_base_manager = options.get('use_base_manager')
        app_dict = self.build_app_dict(app_labels, excludes)
        try:
            file_path = None
            archive = zipfile.ZipFile(output, 'w', zipfile.ZIP_DEFLATED)
            object_iter = self.iter_objects(app_dict, using, use_base_manager,
                                            archive)
            file_handle, file_path = tempfile.mkstemp(suffix='.json',
                                                      prefix='site_dump_')
            datafile = os.fdopen(file_handle, 'wb')
            serializer = JsonSerializer()
            serializer.serialize(object_iter, indent=4, use_natural_keys=True,
                                 stream=datafile)
            datafile.close()
            archive.write(file_path, 'site_dump.json')
            archive.writestr('site_dump_version', '1')
            archive.close()
        except Exception, e:
            if 1 or show_traceback:
                raise
            raise CommandError('Unable to serialize database: %s' % e)
        finally:
            try:
                if file_path:
                    os.remove(file_path)
            except:
                pass



# Python
import datetime
import email.utils
import functools
import hashlib
import os
import shutil
import sys
import StringIO
import tempfile
import zipfile

# Django
from django.test import TestCase
from django.conf import settings
from django.core import mail
from django.core.files.base import ContentFile
from django.core.management import get_commands, load_command_class, BaseCommand
from django.core.management import call_command
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.db import connection, models
from django.db.models import get_model
from django.utils.encoding import is_protected_type
from django.utils import simplejson

# Django-Site-Utils
from site_utils import defaults
from site_utils.serializers import DjangoJSONEncoder
from site_utils.utils import app_is_installed

# TestApp
from models import Document, Photo

# From: http://stackoverflow.com/questions/9882280/find-out-if-a-function-has-been-called
def trackcalls(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        wrapper.has_been_called = True
        return func(*args, **kwargs)
    wrapper.has_been_called = False
    return wrapper

@trackcalls
def site_cleanup_function(**options):
    pass

class TestSiteUtils(TestCase):
    """Test cases for Site Utils management commands and utilities."""

    def setUp(self):
        super(TestSiteUtils, self).setUp()
        self._saved_settings = {}
        for attr in dir(settings):
            if attr == attr.upper():
                self._saved_settings[attr] = getattr(settings, attr)
        settings.MEDIA_ROOT = tempfile.mkdtemp()
        self._temp_dirs = [settings.MEDIA_ROOT]
        self._cwd = os.getcwd()

    def tearDown(self):
        super(TestSiteUtils, self).tearDown()
        for attr in dir(settings):
            if attr == attr.upper() and attr not in self._saved_settings:
                #print 'delete', attr
                delattr(settings, attr)
        for attr, value in self._saved_settings.items():
            if getattr(settings, attr, None) != value:
                #print attr, value
                setattr(settings, attr, value)
        for temp_dir in self._temp_dirs:
            shutil.rmtree(temp_dir, True)
        os.chdir(self._cwd)

    def _call_command(self, name, *args, **options):
        options.setdefault('verbosity', 0)
        original_stdout = sys.stdout
        original_stderr = sys.stderr
        sys.stdout = StringIO.StringIO()
        sys.stderr = StringIO.StringIO()
        result = None
        try:
            #exc_info = None
            result = call_command(name, *args, **options)
        except Exception, e:
            #exc_info = sys.exc_info()
            result = e
        except SystemExit, e:
            #exc_info = sys.exc_info()
            result = e
        finally:
            captured_stdout = sys.stdout.getvalue()
            captured_stderr = sys.stderr.getvalue()
            sys.stdout = original_stdout
            sys.stderr = original_stderr
        #if exc_info:
        #    sys.excepthook(*exc_info)
        return result, captured_stdout, captured_stderr

    def test_app_is_installed(self):
        self.assertTrue(app_is_installed('contenttypes'))
        self.assertTrue(app_is_installed('django.contrib.contenttypes'))
        self.assertTrue(app_is_installed('admin'))
        self.assertTrue(app_is_installed('django.contrib.admin'))
        self.assertFalse(app_is_installed('flatpages'))
        self.assertFalse(app_is_installed('django.contrib.flatpages'))
        self.assertTrue(app_is_installed('site_utils'))

    def test_site_cleanup(self):
        # Create dummy content type.
        self.assertEqual(ContentType.objects.filter(app_label='myapp').count(), 0)
        newct = ContentType.objects.create(name='MyAppModel', app_label='myapp',
                                           model='mymodel')
        self.assertEqual(ContentType.objects.filter(app_label='myapp').count(), 1)
        # Create extra database table.
        all_tables = set(connection.introspection.table_names())
        self.assertFalse('myapp_othermodel' in all_tables)
        cursor = connection.cursor()
        sql = '''CREATE TABLE myapp_othermodel (id INTEGER PRIMARY KEY);'''
        cursor.execute(sql)
        all_tables = set(connection.introspection.table_names())
        self.assertTrue('myapp_othermodel' in all_tables)
        # Run command with dry-run option. Nothing should have changed.
        result = self._call_command('site_cleanup', dry_run=True, verbosity=2)
        self.assertEqual(result[0], None)
        self.assertEqual(ContentType.objects.filter(app_label='myapp').count(), 1)
        all_tables = set(connection.introspection.table_names())
        self.assertTrue('myapp_othermodel' in all_tables)
        # Run command.  Extra content type and table should be gone.
        result = self._call_command('site_cleanup')
        self.assertEqual(result[0], None)
        self.assertEqual(ContentType.objects.filter(app_label='myapp').count(), 0)
        all_tables = set(connection.introspection.table_names())
        self.assertFalse('myapp_othermodel' in all_tables)
        # Change setting to refer to a nonexistent function, which should raise
        # an exception.
        settings.SITE_CLEANUP_FUNCTIONS = ['somepackage.somemodule.somefunction']
        result = self._call_command('site_cleanup')
        self.assertTrue(isinstance(result[0], Exception))
        # Change setting to our own function and verify it gets called.
        settings.SITE_CLEANUP_FUNCTIONS = ['test_project.test_app.tests.site_cleanup_function']
        result = self._call_command('site_cleanup')
        self.assertEqual(result[0], None)
        self.assertEqual(site_cleanup_function.has_been_called, True)

    def test_site_config(self):
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.com')
        self.assertEqual(site.domain, 'example.com')
        # With no options and verbosity=1, should display the current site.
        result = self._call_command('site_config', verbosity=1)
        self.assertEqual(result[0], None)
        self.assertTrue(site.name in result[1])
        self.assertTrue(site.domain in result[1])
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.com')
        self.assertEqual(site.domain, 'example.com')
        # Change only the name.
        result = self._call_command('site_config', _name='example.net')
        self.assertEqual(result[0], None)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.net')
        self.assertEqual(site.domain, 'example.com')
        # Change only the domain.
        result = self._call_command('site_config', domain='example.net')
        self.assertEqual(result[0], None)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.net')
        self.assertEqual(site.domain, 'example.net')
        # Change both name and domain.
        result = self._call_command('site_config', _name='example.org',
                                    domain='example.org')
        self.assertEqual(result[0], None)
        site = Site.objects.get_current()
        self.assertEqual(site.name, 'example.org')
        self.assertEqual(site.domain, 'example.org')

    def _call_site_dump(self, *app_labels, **options):
        options.setdefault('verbosity', 1)
        temp_dir = tempfile.mkdtemp()
        self._temp_dirs.append(temp_dir)
        os.chdir(temp_dir)
        result = self._call_command('site_dump', *app_labels, **options)
        self.assertEqual(result[0], None)
        output = options.get('output', None) or os.listdir(temp_dir)[0]
        if not os.path.isabs(output):
            output = os.path.join(temp_dir, output)
        return result, output

    def _check_site_dump_file(self, output, only_models=None):
        only_models = set(only_models or [])
        seen_models = set()
        archive = zipfile.ZipFile(output, 'r')
        version = archive.read('site_dump_version')
        self.assertEqual(version, '1')
        jsondata = archive.read('site_dump.json')
        data = simplejson.loads(jsondata)
        for objdict in data:
            model = get_model(*objdict['model'].split('.'))
            if only_models:
                self.assertTrue(model in only_models)
            seen_models.add(model)
            obj = model.objects.get(pk=objdict['pk'])
            if 'nk' in objdict:
                obj = model.objects.get_by_natural_key(*objdict['nk'])
            for field_name, field_value in objdict['fields'].items():
                field = obj._meta.get_field(field_name)
                if field in obj._meta.many_to_many:
                    for value in field_value:
                        if isinstance(value, (list, tuple)):
                            relobj = field.rel.to.objects.get_by_natural_key(*value)
                        else:
                            relobj = field.rel.to.get(pk=value)
                elif field.rel:
                    if isinstance(field_value, (list, tuple)):
                        if field_value and isinstance(field_value[0], (list, tuple)):
                            value = field_value[0]
                        else:
                            value = field_value
                        relobj = field.rel.to.objects.get_by_natural_key(*value)
                    else:
                        relobj = field.rel.to.get(pk=field_value)
                else:
                    value = getattr(obj, field_name)
                    if not is_protected_type(value):
                        value = field.value_to_string(obj)
                    if isinstance(value, (datetime.datetime, datetime.date, datetime.time)):
                        # FIXME: Probably a better way, but this gets it done for now.
                        value = simplejson.loads(simplejson.dumps(value, cls=DjangoJSONEncoder))
                    if isinstance(field, models.FileField):
                        field_file = getattr(obj, field_name)
                        field_file.open('rb')
                        file_md5 = hashlib.md5()
                        while True:
                            data = field_file.read(2**17)
                            if not data:
                                break
                            file_md5.update(data)
                        archive_name = 'site_media/%s' % value
                        if hasattr(archive, 'open'):
                            archive_file = archive.open(archive_name, 'r')
                        else:
                            archive_file = StringIO.StringIO(archive.read(archive_name))
                        archive_md5 = hashlib.md5()
                        while True:
                            data = archive_file.read(2**17)
                            if not data:
                                break
                            archive_md5.update(data)
                        self.assertEqual(file_md5.hexdigest(), archive_md5.hexdigest())
                    self.assertEqual(value, field_value)
        if only_models:
            self.assertEqual(only_models, seen_models)

    def test_site_dump(self):
        # Test default with no options.
        result, output = self._call_site_dump()
        self._check_site_dump_file(output)
        # Create user/group data (adds many to many relation).
        group1 = Group.objects.create(name='GroupOne')
        group2 = Group.objects.create(name='GroupTwo')
        user = User.objects.create_superuser('superfred', 'fred@ixmm.net', 'fredsPASS')
        user.first_name = 'Super'
        user.last_name = 'Fred'
        user.save()
        user.groups.add(group1)
        user.groups.add(group2)
        result, output = self._call_site_dump()
        self._check_site_dump_file(output)
        # Create models with attached files (also date and time fields).
        document = Document.objects.create(title='Test Doc', created_by=user)
        document.doc.save('test.txt', ContentFile('this is some test stuff'))
        photo = Photo.objects.create(caption='Test Photo', created_by=user)
        photo.image.save('test.jpg', ContentFile('blah blah blah'))
        result, output = self._call_site_dump()
        self._check_site_dump_file(output)
        # Specify only certain apps/models.
        result, output = self._call_site_dump('test_app')
        self._check_site_dump_file(output, (Document, Photo))
        result, output = self._call_site_dump('test_app.Document')
        self._check_site_dump_file(output, (Document,))
        # Exclude specific apps/models.
        result, output = self._call_site_dump('auth', exclude=['auth.Permission'])
        self._check_site_dump_file(output, (User, Group))
        result, output = self._call_site_dump(exclude=['south', 'admin'])
        self._check_site_dump_file(output, (User, Group, Permission, ContentType,
                                            Site, Document, Photo))
        # Specify alternate output file name.
        result, output = self._call_site_dump(output='blah.zip')
        self.assertEqual(os.path.basename(output), 'blah.zip')
        self._check_site_dump_file(output)

    #def test_site_load(self):
    #    raise NotImplementedError

    def test_site_notify(self):
        # Create superuser.
        super_user = User.objects.create_superuser('superfred', 'fred@ixmm.net', 'fredsPASS')
        super_user.first_name = 'Super'
        super_user.last_name = 'Fred'
        super_user.save()
        # Create inactive superuser.
        inactive_super_user = User.objects.create_superuser('deadfred', 'fred2@ixmm.net', 'fredsPASS')
        inactive_super_user.first_name = 'Dead'
        inactive_super_user.last_name = 'Fred'
        inactive_super_user.is_active = False
        inactive_super_user.save()
        # Create staff user.
        staff_user = User.objects.create_user('staffbob', 'bob@ixmm.net', 'bobsPASS')
        staff_user.first_name = 'Staff'
        staff_user.last_name = 'Bob'
        staff_user.is_staff = True
        staff_user.save()
        # Create staff user who is also in settings.MANAGERS.
        manager_staff_user = User.objects.create_user('david', 'david@ninemoreminutes.com', 'davesPASS')
        manager_staff_user.first_name = 'David'
        manager_staff_user.last_name = 'Horton'
        manager_staff_user.is_staff = True
        manager_staff_user.save()
        self.assertEqual(len(mail.outbox), 0)
        # Send to default recipients (admins).
        result = self._call_command('site_notify')
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 1)
        msg = mail.outbox[-1]
        expected_emails = set([x[1] for x in settings.ADMINS])
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Send to managers only.
        result = self._call_command('site_notify', managers=True)
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 2)
        msg = mail.outbox[-1]
        expected_emails = set([x[1] for x in settings.MANAGERS])
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Send to active superusers only.
        result = self._call_command('site_notify', superusers=True)
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 3)
        msg = mail.outbox[-1]
        users = User.objects.filter(is_active=True, is_superuser=True)
        expected_emails = set(users.values_list('email', flat=True))
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Send to active staff only.
        result = self._call_command('site_notify', staff=True)
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 4)
        msg = mail.outbox[-1]
        users = User.objects.filter(is_active=True, is_staff=True)
        expected_emails = set(users.values_list('email', flat=True))
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Send to managers and active staff. Email address in both lists should
        # only be listed once.
        result = self._call_command('site_notify', managers=True, staff=True)
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 5)
        msg = mail.outbox[-1]
        users = User.objects.filter(is_active=True, is_staff=True)
        expected_emails = set(users.values_list('email', flat=True))
        expected_emails.update([x[1] for x in settings.MANAGERS])
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Send to all admins, managers and staff.
        result = self._call_command('site_notify', all=True)
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 6)
        msg = mail.outbox[-1]
        users = User.objects.filter(is_active=True, is_staff=True)
        expected_emails = set(users.values_list('email', flat=True))
        users = User.objects.filter(is_active=True, is_superuser=True)
        expected_emails.update(users.values_list('email', flat=True))
        expected_emails.update([x[1] for x in settings.MANAGERS])
        expected_emails.update([x[1] for x in settings.ADMINS])
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Send to default recipients (admins) bcc'ed.
        result = self._call_command('site_notify', bcc=True)
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 7)
        msg = mail.outbox[-1]
        expected_emails = set([x[1] for x in settings.ADMINS])
        self.assertFalse(msg.to)
        for recipient in msg.bcc:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Change default recipients via setting.
        settings.SITE_NOTIFY_DEFAULT_RECIPIENTS = ('admins', 'managers')
        result = self._call_command('site_notify')
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 8)
        msg = mail.outbox[-1]
        expected_emails = set([x[1] for x in settings.ADMINS])
        expected_emails.update([x[1] for x in settings.MANAGERS])
        for recipient in msg.to:
            realname, email_address = email.utils.parseaddr(recipient)
            self.assertTrue(email_address in expected_emails)
            expected_emails.remove(email_address)
        self.assertFalse(expected_emails)
        # Positional arguments should become message subject, then body.
        result = self._call_command('site_notify', 'test_subject', 'test_body', 'test_body2')
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 9)
        msg = mail.outbox[-1]
        self.assertTrue('test_subject' in msg.subject)
        self.assertTrue('test_body' in msg.body)
        self.assertTrue('test_body2' in msg.body)
        # Override subject and body templates via command line arguments.
        result = self._call_command('site_notify', subject_template='new_site_notify_subject.txt', body_template='new_site_notify_body.txt')
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 10)
        msg = mail.outbox[-1]
        self.assertTrue('NEW_SUBJECT' in msg.subject)
        self.assertTrue('NEW_BODY_SUFFIX' in msg.body)
        # Override subject and body templates via settings.
        settings.SITE_NOTIFY_SUBJECT_TEMPLATE = 'new_site_notify_subject.txt'
        settings.SITE_NOTIFY_BODY_TEMPLATE = 'new_site_notify_body.txt'
        result = self._call_command('site_notify')
        self.assertEqual(result[0], None)
        self.assertEqual(len(mail.outbox), 11)
        msg = mail.outbox[-1]
        self.assertTrue('NEW_SUBJECT' in msg.subject)
        self.assertTrue('NEW_BODY_SUFFIX' in msg.body)

    def _get_command_class(self, name):
        app_name = get_commands()[name]
        if isinstance(app_name, BaseCommand):
            instance = app_name
        else:
            instance = load_command_class(app_name, name)
        return type(instance)

    def _track_command_class(self, name):
        klass = self._get_command_class(name)
        klass.execute = trackcalls(klass.execute)
        self.assertFalse(klass.execute.has_been_called)

    def assertCommandCalled(self, name):
        klass = self._get_command_class(name)
        self.assertTrue(getattr(klass.execute, 'has_been_called', None))

    def assertCommandNotCalled(self, name):
        klass = self._get_command_class(name)
        self.assertFalse(getattr(klass.execute, 'has_been_called', None))

    def test_site_update(self):
        # Run site update with built-in default options, verify only expected
        # commands have been called.
        self._track_command_class('validate')
        self._track_command_class('syncdb')
        self._track_command_class('migrate')
        self._track_command_class('collectstatic')
        self._track_command_class('cleanup')
        result = self._call_command('site_update', interactive=False)
        self.assertEqual(result[0], None)
        self.assertCommandNotCalled('validate')
        self.assertCommandCalled('syncdb')
        self.assertCommandCalled('migrate')
        self.assertCommandCalled('collectstatic')
        self.assertCommandNotCalled('cleanup')
        # Replace the default update commands via settings.
        settings.SITE_UPDATE_COMMANDS = {
            'default': [
                'validate',
                'syncdb',
                'migrate',
                ('collectstatic', (), {'clear': True}),
            ],
            'clean': [
                'cleanup',
            ],
            'blah': [
                ('blah', (), {}, 'blahapp'),
            ],
            'argh': [
                'argh',
            ],
            'wtf': [
                (),
            ],
            'wtf2': [
                None,
            ],
        }
        # Run again and verify that the updated list of 'default' commands was
        # used.
        self._track_command_class('validate')
        self._track_command_class('syncdb')
        self._track_command_class('migrate')
        self._track_command_class('collectstatic')
        self._track_command_class('cleanup')
        result = self._call_command('site_update', interactive=False)
        self.assertEqual(result[0], None)
        self.assertCommandCalled('validate')
        self.assertCommandCalled('syncdb')
        self.assertCommandCalled('migrate')
        self.assertCommandCalled('collectstatic')
        self.assertCommandNotCalled('cleanup')
        # Run again using 'clean' subcommand, verify only the expected command
        # was called.
        self._track_command_class('validate')
        self._track_command_class('syncdb')
        self._track_command_class('migrate')
        self._track_command_class('collectstatic')
        self._track_command_class('cleanup')
        result = self._call_command('site_update', 'clean', interactive=False)
        self.assertEqual(result[0], None)
        self.assertCommandNotCalled('validate')
        self.assertCommandNotCalled('syncdb')
        self.assertCommandNotCalled('migrate')
        self.assertCommandNotCalled('collectstatic')
        self.assertCommandCalled('cleanup')
        # Run again using 'default' and 'clean' subcommands, verify that all
        # commands were called.
        self._track_command_class('validate')
        self._track_command_class('syncdb')
        self._track_command_class('migrate')
        self._track_command_class('collectstatic')
        self._track_command_class('cleanup')
        result = self._call_command('site_update', 'default', 'clean', interactive=False)
        self.assertEqual(result[0], None)
        self.assertCommandCalled('validate')
        self.assertCommandCalled('syncdb')
        self.assertCommandCalled('migrate')
        self.assertCommandCalled('collectstatic')
        self.assertCommandCalled('cleanup')
        # Run again using 'blah' subcommand, should still succeed since
        # 'blahapp' is specified and not installed.
        self._track_command_class('validate')
        self._track_command_class('syncdb')
        self._track_command_class('migrate')
        self._track_command_class('collectstatic')
        self._track_command_class('cleanup')
        result = self._call_command('site_update', 'blah', interactive=False)
        self.assertEqual(result[0], None)
        self.assertCommandNotCalled('validate')
        self.assertCommandNotCalled('syncdb')
        self.assertCommandNotCalled('migrate')
        self.assertCommandNotCalled('collectstatic')
        self.assertCommandNotCalled('cleanup')
        # Run again using 'argh' subcommand, should fail since 'argh' is an
        # unknown command.
        self._track_command_class('validate')
        self._track_command_class('syncdb')
        self._track_command_class('migrate')
        self._track_command_class('collectstatic')
        self._track_command_class('cleanup')
        result = self._call_command('site_update', 'argh', interactive=False)
        self.assertTrue(isinstance(result[0], SystemExit))
        self.assertCommandNotCalled('validate')
        self.assertCommandNotCalled('syncdb')
        self.assertCommandNotCalled('migrate')
        self.assertCommandNotCalled('collectstatic')
        self.assertCommandNotCalled('cleanup')
        # Run again using 'wtf' subcommand with invalid commands, should fail.
        result = self._call_command('site_update', 'wtf', interactive=False)
        self.assertTrue(isinstance(result[0], SystemExit))
        # Run again using 'wtf2' subcommand with invalid commands, should fail.
        result = self._call_command('site_update', 'wtf2', interactive=False)
        self.assertTrue(isinstance(result[0], SystemExit))
        # Run again using unknown subcommand, should fail.
        result = self._call_command('site_update', 'hoohah', interactive=False)
        self.assertTrue(isinstance(result[0], SystemExit))
